"""
自动生成接口文档
"""
import importlib
import inspect
import os
import re

from docxtpl import DocxTemplate
from werkzeug.utils import find_modules

from framework.interface.abstract_api import AbstractApi
from framework.the_path import ThePath


def get_friendly_type(type):
    types = {
        'int': '整数',
        'str': '字符串',
        'float': '小数',
        'decimal': '小数',
    }
    return types.get(type, '字符串')


def _build_class_info_item(method, rule_data:dict, urls, cls_name, cls, title):
    params = []
    for k, param in rule_data.items():
        param['name'] = k
        param['_type'] = get_friendly_type(param['type'])
        param['required'] = '必须' if param['required'] else ''
        params.append(param)
    return {'url': urls, 'cls_name': cls_name, 'cls': cls, 'title': title, 'method': method, 'params': params}


def _is_function_override(func):
    return not str(func.__module__).endswith('abstract_api')


def get_class_info():
    class_info = []
    modules = find_modules('api.app', recursive=True)
    for mod in modules:
        package_name = mod.split('.')[2]
        module = importlib.import_module(mod)
        cls_list = inspect.getmembers(module, inspect.isclass)

        for cls_name, cls in cls_list:
            if cls != AbstractApi and issubclass(cls, AbstractApi):
                if not cls.url_names():
                    slug = re.compile(r'([a-z]|\d)([A-Z])').sub(r'\1-\2', cls_name).lower()
                    urls = f"/{package_name}/{slug}"
                else:
                    slugs = cls.url_names()
                    urls = [f"/{package_name}/{slug}" for slug in slugs]

                title = cls.__doc__.strip()

                if _is_function_override(cls.handle_get_request):
                    class_info.append(_build_class_info_item('GET', cls.get_param_rules, urls, cls_name, cls, title))
                if _is_function_override(cls.handle_post_request):
                    class_info.append(_build_class_info_item('POST', cls.get_param_rules, urls, cls_name, cls, title))

    return class_info


def api_doc(project: str):
    # FIXME 同一个接口有多个method，只能生成一个method的文档
    template_file = os.path.join(ThePath.doc(), 'tpl/api_template.docx')
    tpl = DocxTemplate(template_file)
    tpl.render({'project': project, 'items': get_class_info()})

    save_file_path = os.path.join(ThePath.tmp('doc'), f'{project}.docx')
    tpl.save(save_file_path)


if __name__ == '__main__':
    api_doc('Yard-base演示项目')
