# -*- coding:utf-8 -*-
# @Time   : 2023/7/5 9:37
# @Author : Fushuyue
# @Owner  : YSRD (Insights Value)

import pandas as pd


def calc_total_duration(warn_df):
    days, hours, minutes, max_duration, max_duration_data = 0, 0, 0, 0, None

    def split_duration(x):
        nonlocal days, hours, minutes, max_duration, max_duration_data
        x_duration_minutes = 0
        duration = x['duration']
        if '天' in duration:
            x_list = duration.split('天')
            days += int(x_list[0])
            x_duration_minutes += int(x_list[0]) * 24 * 60
            duration = x_list[1]
        if '小时' in duration:
            x_list = duration.split('小时')
            hours += int(x_list[0])
            x_duration_minutes += int(x_list[0]) * 60
            duration = x_list[1]
        if '分' in duration:
            x_list = duration.split('分')
            minutes += int(x_list[0])
            x_duration_minutes += int(x_list[0])
        # 找出持续时长最大的一条报警
        if x_duration_minutes > max_duration:
            max_duration_data = x.to_dict()
            max_duration = x_duration_minutes

    warn_df.apply(func=split_duration, axis=1)
    hours += minutes // 60
    minutes = minutes % 60
    days += hours // 24
    hours = hours % 24

    duration = f'{minutes}分'
    if hours:
        duration = f'{hours}小时' + duration
    if days:
        duration = f'{days}天' + duration

    return duration, max_duration_data


def format_data(warn_list):
    if not warn_list:
        return list()
    # 分组统计每个点位的：报警类型、最高报警级别、报警总时长、报警次数
    warn_level_map = {1: '一级', 2: '二级', 3: '三级'}
    res = list()
    for key, val_df in pd.DataFrame(warn_list).groupby(
            by=["ent_id", "dev_id", "dev_name", "warn_type_id", "warn_type_name"]):
        total_duration, max_duration_data = calc_total_duration(val_df)  # 计算报警总持续时长和持续时长最大的一条报警
        res.append({
            'ent_id': 0 if not key[0] else int(key[0]),
            'dev_id': 0 if not key[1] else int(key[1]),
            'dev_name': str(key[2]),  # 报警设备
            'warn_type_id': 0 if not key[3] else int(key[3]),  # 报警类型
            'warn_type_name': str(key[4]),  # 报警类型
            'warn_count': len(val_df),  # 报警次数
            'highest_level': warn_level_map[int(val_df['warn_level'].min())] if val_df['warn_level'].min() else 0,  # 报警最高等级(一级为最高)
            'total_duration': total_duration,
            'warn_code': [str(i) for i in val_df['warn_code'].values],
            'start_time': str(val_df['warn_time'].min()),  # 最早开始报警时间
            'end_time': str(val_df['end_time'].max()),  # 最晚结束报警时间
            'warn_data': sorted(val_df.to_dict(orient='records'), key=lambda x: x['warn_time'], reverse=True),
            'max_duration_data': max_duration_data  # 最大持续时长的一条报警信息
        })
    return res
