import json
import requests
import xml.dom.minidom as xmldom
import os
import uuid
import zipfile
import shutil
from openpyxl import load_workbook



def isfile_exist(file_path):
    if not os.path.isfile(file_path):
        print("It's not a file or no such file exist ! %s" % file_path)
        return False
    else:
        return True


# 复制并修改指定目录下的文件类型名，将excel后缀名修改为.zip
def copy_change_file_name(file_path, new_type='.zip'):
    if not isfile_exist(file_path):
        return ''
    extend = os.path.splitext(file_path)[1]  # 获取文件拓展名
    if extend != '.xlsx' and extend != '.xls':
        print("It's not a excel file! %s" % file_path)
        return False
    file_name = os.path.basename(file_path)  # 获取文件名
    new_name = str(file_name.split('.')[0]) + new_type  # 新的文件名，命名为：xxx.zip
    dir_path = os.path.dirname(file_path)  # 获取文件所在目录
    new_path = os.path.join(dir_path, new_name)  # 新的文件路径
    if os.path.exists(new_path):
        os.remove(new_path)
    return shutil.copyfile(file_path, new_path)     # 返回新的文件路径，压缩包

# 解压文件
def unzip_file(zipfile_path):
    if not isfile_exist(zipfile_path):
        return False
    if os.path.splitext(zipfile_path)[1] != '.zip':
        print("It's not a zip file! %s" % zipfile_path)
        return False
    file_zip = zipfile.ZipFile(zipfile_path, 'r')
    file_name = os.path.basename(zipfile_path)  # 获取文件名
    zipdir = os.path.join(os.path.dirname(zipfile_path), str(file_name.split('.')[0]))  # 获取文件所在目录
    for files in file_zip.namelist():
        file_zip.extract(files, zipdir)  # 解压到指定文件目录
    file_zip.close()
    return zipdir

# 读取解压后的文件夹，打印图片路径
def read_img(unzip_file_path):
    img_dict = dict()
    pic_dir = 'xl' + os.sep + 'media'  # excel变成压缩包后，再解压，图片在media目录
    pic_path = os.path.join(unzip_file_path, pic_dir)     # 解压后图片所在绝对目录
    file_list = os.listdir(pic_path)     # 找到图片目录下的所有图片名称
    for file in file_list:
        # img_url = os.path.join(pic_path, file)   # 获取图片本地完整路径
        img_url = file_upload(img_path=os.path.join(pic_path, file))   # 将图片保存到服务服务器上
        img_dict[file] = img_url
    return img_dict

# 保存数据到服务器
def file_upload(img_path):
    data = {'subsystem_code': 'electricity-service', 'subsystem_api': 'elec-picture'}
    file_name = str(uuid.uuid4()) + '.' + img_path.split('.')[-1]
    try:
        with open(img_path, 'rb') as f:
            r = requests.post(url="https://api-file-manage.airqualitychina.cn:9998/api/hold-filename-upload",
                              data=data,
                              files=[('files', (file_name, f.read()))])
        file_url = "https://f.hotgrid.cn/" + json.loads(r.content)['result'][0]['url']
        print(file_url)
    except Exception as e:
        print(e)
        file_url = ''
    return file_url

def get_img_pos_info(unzip_file_path, img_dict, drawing_idx):
    """解析xml 文件，获取图片在excel表格中的索引位置信息"""
    # 1. 查询rid与图片的关联关系
    xml_rel_dir = 'xl' + os.sep + 'drawings' + os.sep + '_rels' + os.sep + f'drawing{drawing_idx}.xml.rels'
    xml_rel_path = os.path.join(unzip_file_path, xml_rel_dir)
    img_info = parse_xml_rel(xml_rel_path)

    # 2. 解析xml 文件， 返回图片索引位置信息
    xml_dir = 'xl' + os.sep + 'drawings' + os.sep + f'drawing{drawing_idx}.xml'
    xml_path = os.path.join(unzip_file_path, xml_dir)
    image_info_dict = parse_xml(xml_path, img_dict, img_info)
    return image_info_dict

# 解析xml关联文件并获取对应图片位置
def parse_xml_rel(file_name):
    # 得到文档对象
    image_info = dict()
    dom_obj = xmldom.parse(file_name)
    # 得到元素对象
    element = dom_obj.documentElement
    relationship = element.getElementsByTagName("Relationship")
    for rel in relationship:
        embed = rel.getAttribute('Id')
        img = rel.getAttribute('Target').replace('../media/', '')
        image_info[embed] = img
    return image_info



# 解析xml文件并获取对应图片位置
def parse_xml(file_name, img_dict, img_info):
    # 得到文档对象
    new_image_info = dict()
    dom_obj = xmldom.parse(file_name)
    # 得到元素对象
    element = dom_obj.documentElement
    sub_twoCellAnchor = element.getElementsByTagName("xdr:twoCellAnchor")
    if len(sub_twoCellAnchor) == 0:
        sub_twoCellAnchor = element.getElementsByTagName("xdr:oneCellAnchor")

    for anchor in sub_twoCellAnchor:
        # xdr_from = anchor.getElementsByTagName('xdr:from')[0]
        try:
            col = anchor.getElementsByTagName('xdr:col')[0].firstChild.nodeValue  # 获取标签间的数据
            row = anchor.getElementsByTagName('xdr:row')[0].firstChild.nodeValue
            embed = anchor.getElementsByTagName('a:blip')[0].getAttribute('r:embed')  # 获取属性
            # col = xdr_from.childNodes[0].firstChild.data  # 获取标签间的数据
            # row = xdr_from.childNodes[2].firstChild.data
            # embed = \
            #     anchor.getElementsByTagName('xdr:pic')[0].getElementsByTagName('xdr:blipFill')[0].getElementsByTagName(
            #         'a:blip')[0].getAttribute('r:embed')  # 获取属性
            if (int(row) + 1, int(col) + 1) in new_image_info:
                new_image_info[(int(row) + 1, int(col) + 1)] += ',' + img_dict.get(img_info.get(embed, ''), '')
            else:
                new_image_info[(int(row) + 1, int(col) + 1)] = img_dict.get(img_info.get(embed, ''), '')
        except Exception as e:
            print(e)

    return new_image_info

def parse_image(file):
    # 先将文件保存到本地
    file_path = os.path.join(os.getcwd(), file.filename)
    file.save(file_path)
    # 0. 获取压缩包文件路径
    zip_file_path = copy_change_file_name(file_path)
    if zip_file_path != '':
        # 1. 获取解压文件路径
        unzip_file_path = unzip_file(zip_file_path)
        if isinstance(unzip_file_path, str):
            # 2. 获取解压目录下所有的图片信息
            img_dict = read_img(unzip_file_path)
            import pandas as pd
            df = pd.DataFrame(list(img_dict))
            # 3. 加载excel文件
            book = load_workbook(file_path)
            drawing_idx = 1   # 用于累计drawing(i).xml.rels的数量
            # 4. 将每个sheet中的图片替换为图片的url
            for sheet_name in book.sheetnames:
                sheet = book[sheet_name]
                # 判断sheet中是否有图片，若有再替换
                if hasattr(sheet, '_images') and sheet._images:
                    img_info_dict = get_img_pos_info(unzip_file_path, img_dict, drawing_idx=drawing_idx)
                    for key, val in img_info_dict.items():
                        sheet.cell(row=key[0], column=key[1], value=val)
                    drawing_idx += 1

            # 删除旧的excel文件，保存替换图片url后新的excel文件
            if os.path.exists(file_path):
                os.remove(file_path)
            book.save(file_path)

            # 删除解压路径
            if os.path.exists(unzip_file_path):
                shutil.rmtree(unzip_file_path)

        # 删除压缩包
        if os.path.exists(zip_file_path):
            os.remove(zip_file_path)

    return file_path

if __name__ == '__main__':
    # res = parse_image(
    #     file_path='C:\\Users\\admin\\Desktop\\reproject\\electricity-api-service\\src\\app\\v1\\file_upload\\顺义区用电企业数据问题(3).xlsx')
    print(os.getcwd())

    file_path = r'C:\Users\admin\Desktop\reproject\electricity-api-service\src\用电项目后台数据库录入用表.xlsx'
