# @Time   : 2022-03-03 17:00:00
# @Author : Gavin Jiang
# @Owner  : YSRD (Insights Value)

from enum import Enum, unique


@unique
class RespCode(Enum):
    """
    状态码常量类

    系统错误: 1000
    正常：2000
    通用异常：3000
    数据异常：4000
    """

    INNER_ERROR = 1001
    PLEASE_TRY_LATER = 1002

    OK = 2000

    TOKEN_INVALID = 3001
    TOKEN_EXPIRED = 3002
    PARAM_LACK = 3003
    PARAM_WRONG_FORMAT = 3004
    UNAUTHORIZED = 3005
    POST_REPEAT = 3007
    REQUEST_METHOD_INVALID = 3008
    INVALID_PARAM = 3009
    NOT_SUPPORT = 3010
    NOT_FOUND_DATA = 3011
    INVALID_OPERATION = 3012
    INVALID_JSON_FORMAT = 3013

    DATA_ERROR = 4001
    NOT_FOUND_USER = 4004

    def desc(self):
        return _RespCodeDesc.desc_dict.get(self)


class _RespCodeDesc:
    desc_dict = {
        RespCode.INNER_ERROR: '内部错误',
        RespCode.PLEASE_TRY_LATER: '操作失败，请稍后重试',

        RespCode.OK: "访问成功",

        RespCode.TOKEN_INVALID: '非法的token',
        RespCode.TOKEN_EXPIRED: 'token已失效',
        RespCode.PARAM_LACK: '缺失参数',
        RespCode.PARAM_WRONG_FORMAT: '参数格式不正确',
        RespCode.UNAUTHORIZED: '未授权',
        RespCode.POST_REPEAT: '重复提交',
        RespCode.REQUEST_METHOD_INVALID: '非法的请求方式',
        RespCode.INVALID_PARAM: '非法的请求参数',
        RespCode.NOT_SUPPORT: '不支持该操作',
        RespCode.NOT_FOUND_DATA: '找不到数据',
        RespCode.INVALID_OPERATION: '无效操作',
        RespCode.INVALID_JSON_FORMAT: '错误的JSON格式',

        RespCode.DATA_ERROR: '数据错误',

        RespCode.NOT_FOUND_USER: 'user_code 未知'
    }
